@@ -123,6 +123,7 @@ enum {
         PROP_UDA_VERSION,
         PROP_BOOT_ID,
         PROP_CONFIG_ID,
+        PROP_PORT,
 };
 
 enum {
@@ -333,6 +334,7 @@ gssdp_client_get_property (GObject    *object,
                 g_value_set_uint (value, priv->socket_ttl);
                 break;
         case PROP_MSEARCH_PORT:
+        case PROP_PORT:
                 g_value_set_uint (value, priv->msearch_port);
                 break;
         case PROP_ADDRESS_FAMILY:
@@ -392,6 +394,7 @@ gssdp_client_set_property (GObject      *object,
                 priv->socket_ttl = g_value_get_uint (value);
                 break;
         case PROP_MSEARCH_PORT:
+        case PROP_PORT:
                 priv->msearch_port = g_value_get_uint (value);
                 break;
         case PROP_ADDRESS_FAMILY:
@@ -602,6 +605,8 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          * UDP port to use for sending multicast M-SEARCH requests on the
          * network. If not set (or set to 0) a random port will be used.
          * This property can be only set during object construction.
+         *
+         * Deprecated: 1.6.0: Use GSSDPClient:port instead
          */
         g_object_class_install_property
                 (object_class,
@@ -616,6 +621,27 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                          G_PARAM_CONSTRUCT_ONLY |
                          G_PARAM_STATIC_STRINGS));
 
+        /**
+         * GSSDPClient:port:
+         *
+         * UDP port to use for sending multicast M-SEARCH requests on the
+         * network. If not set (or set to 0) a random port will be used.
+         * This property can be only set during object construction.
+         *
+         * Deprecated: 1.6.0
+         */
+        g_object_class_install_property
+                (object_class,
+                 PROP_PORT,
+                 g_param_spec_uint
+                 ("port",
+                  "M-SEARCH port",
+                  "UDP port to use for M-SEARCH requests",
+                  0, G_MAXUINT16,
+                  0,
+                  G_PARAM_READWRITE |
+                          G_PARAM_CONSTRUCT_ONLY |
+                          G_PARAM_STATIC_STRINGS));
         /**
          * GSSDPClient:address-family:(attributes org.gtk.Property.get=gssdp_client_get_family):
          *
@@ -864,8 +890,9 @@ gssdp_client_get_host_ip (GSSDPClient *client)
         priv = gssdp_client_get_instance_private (client);
 
         if (priv->device.host_ip == NULL)
-                priv->device.host_ip = g_inet_address_to_string
-                                    (priv->device.host_addr);
+                if (priv->device.host_addr != NULL)
+                        priv->device.host_ip = g_inet_address_to_string
+                                            (priv->device.host_addr);
 
         return priv->device.host_ip;
 }
@@ -1287,8 +1314,7 @@ gssdp_client_can_reach (GSSDPClient *client, GInetSocketAddress *address)
         GSSDPClientPrivate *priv = gssdp_client_get_instance_private (client);
 
         GInetAddress *addr = g_inet_socket_address_get_address (address);
-        if (g_inet_address_get_is_link_local (addr) &&
-            g_inet_address_get_family (addr) == G_SOCKET_FAMILY_IPV6) {
+        if (g_inet_address_get_is_link_local (addr)) {
                 return g_inet_socket_address_get_scope_id (address) ==
                        priv->device.index;
         }
@@ -1296,6 +1322,16 @@ gssdp_client_can_reach (GSSDPClient *client, GInetSocketAddress *address)
         return g_inet_address_mask_matches (priv->device.host_mask, addr);
 }
 
+guint
+gssdp_client_get_port (GSSDPClient *client)
+{
+        g_return_val_if_fail (GSSDP_IS_CLIENT (client), 0);
+
+        GSSDPClientPrivate *priv = gssdp_client_get_instance_private (client);
+
+        return priv->msearch_port;
+}
+
 /**
  * _gssdp_client_send_message:
  * @client: A #GSSDPClient
@@ -1498,7 +1534,7 @@ parse_http_request (char                *buf,
 
                 return TRUE;
         } else {
-                soup_message_headers_free (*headers);
+                soup_message_headers_unref (*headers);
                 *headers = NULL;
 
                 g_free (path);
@@ -1529,8 +1565,7 @@ parse_http_response (char                *buf,
 
                 return TRUE;
         } else {
-                soup_message_headers_free (*headers);
-                *headers = NULL;
+                g_clear_pointer (headers, soup_message_headers_unref);
 
                 return FALSE;
         }
@@ -1607,29 +1642,19 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
                                 msg = GSSDP_PKTINFO6_MESSAGE (messages[i]);
                                 msg_ifindex = gssdp_pktinfo6_message_get_ifindex (msg);
                                 local_addr = gssdp_pktinfo6_message_get_local_addr (msg);
-                        } else {
+                        } else
                                 continue;
-                        }
 
                         /* message needs to be on correct interface or on
                          * loopback (as kernel can be smart and route things
                          * there even if sent to another network) */
-                        if (g_inet_address_equal (local_addr, group_addr)) {
-                                // This is a multicast packet. If the index is not our index, ignore
-                                if (msg_ifindex != priv->device.index) {
-                                        goto out;
-                                }
-                                break;
-                        }
-
-                        if (g_inet_address_equal (local_addr,
-                                                  priv->device.host_addr)) {
-                                // This is a "normal" packet. We can receive those
-
-                                if (msg_ifindex != priv->device.index &&
-                                    msg_ifindex != LOOPBACK_IFINDEX) {
-                                        goto out;
-                                }
+                        if (!((msg_ifindex == priv->device.index ||
+                               msg_ifindex == LOOPBACK_IFINDEX) &&
+                              (g_inet_address_equal (local_addr,
+                                                     priv->device.host_addr) ||
+                               g_inet_address_equal (local_addr, group_addr)))) {
+                                goto out;
+                        } else {
                                 break;
                         }
                 }
@@ -1720,7 +1745,7 @@ out:
 
         g_free (ip_string);
 
-        g_clear_pointer (&headers, soup_message_headers_free);
+        g_clear_pointer (&headers, soup_message_headers_unref);
         g_clear_object (&address);
 
         if (messages) {
