@@ -345,10 +345,9 @@ gssdp_socket_source_do_init (GInitable                   *initable,
         success = TRUE;
 
 error:
-        if (bind_address != NULL)
-                g_object_unref (bind_address);
-        if (group != NULL)
-                g_object_unref (group);
+        g_clear_object (&bind_address);
+        g_clear_object (&group);
+
         if (!success)
                 /* Be aware that inner_error has already been free'd by
                  * g_propagate_error(), so we cannot access its contents
@@ -519,7 +518,7 @@ gssdp_socket_source_class_init (GSSDPSocketSourceClass *klass)
                         ("index",
                          "Interface index",
                          "Interface index of the network device",
-                         -1, G_MAXINT,
+                         -1, G_MAXUINT16,
                          -1,
                          G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY |
                          G_PARAM_STATIC_STRINGS));
